@@ -7,21 +7,27 @@ Usage:
   btk pipeline generate-config <ACCESSION> [--coverage 30] [--download]
     [--out /path/to/output/directory] [--db /path/to/database/directory]
     [--db-suffix STRING] [--reads STRING...] [--read-runs INT] [--api-key STRING]
-    [--platforms STRING]
+    [--platforms STRING] [--datasets] [--protocol STRING] 
+    [--download-client STRING] [--retry-times INT]
 
 Options:
-  --coverage=INT         Maximum coverage for read mapping [default: 30]
-  --download             Flag to download remote files [default: False]
-  --out PATH             Path to output directory [default: .]
-  --db PATH              Path to database directory [default: .]
-  --db-suffix STRING     Database version suffix (e.g. 2021_06)
-  --reads STRING         Read accession to include.
-  --read-runs INT        Maximum number of read runs [default: 3]
-  --api-key STRING       NCBI api key for use with edirect
-  --platforms STRING     priority order for sequencing platforms
-                         [default: PACBIO_SMRT,ILLUMINA_XTEN,ILLUMINA,OXFORD_NANOPORE,OTHER]
+  --coverage=INT            Maximum coverage for read mapping [default: 30]
+  --download                Flag to download remote files [default: False]
+  --out PATH                Path to output directory [default: .]
+  --db PATH                 Path to database directory [default: .]
+  --db-suffix STRING        Database version suffix (e.g. 2021_06)
+  --reads STRING            Read accession to include.
+  --read-runs INT           Maximum number of read runs [default: 3]
+  --api-key STRING          NCBI api key for use with edirect
+  --platforms STRING        priority order for sequencing platforms
+                            [default: PACBIO_SMRT,ILLUMINA_XTEN,ILLUMINA,OXFORD_NANOPORE,OTHER]
+  --protocol STRING         Download files using ftp(s) or http(s) [default: ftp]
+  --download-client STRING  Client to use for downloads (aria2c|curl|datasets) [default: curl]
+  --retry-times INT         Number of times to retry a failed download [default: 0]
 """
 
+# pyright: reportMissingModuleSource=false
+
 import gzip
 import os
 import re
@@ -47,7 +53,7 @@ from tolkein import tolog
 
 LOGGER = tolog.logger(__name__)
 
-GOAT_API = "https://goat.genomehubs.org/api/v0.0.1"
+GOAT_API = "https://goat.genomehubs.org/api/v2"
 ENA_API = "https://www.ebi.ac.uk/ena/browser/api"
 GCA_NAME = re.compile(r"GCA_.+")
 BUSCO_URL = "https://busco-data.ezlab.org/v5/data/lineages"
@@ -125,46 +131,80 @@ def find_busco_lineages(ancestors):
         "33090": "viridiplantae",
         "71240": "eudicots",
     }
-    # line = grep_line(lineage_file, taxid)
-    lineages = []
-    for obj in ancestors:
-        if obj["taxon_id"] in BUSCO_SETS:
-            lineages.append("%s_odb10" % BUSCO_SETS[obj["taxon_id"]])
-    lineages.append("bacteria_odb10")
-    lineages.append("archaea_odb10")
+    lineages = [
+        f'{BUSCO_SETS[obj["taxon_id"]]}_odb10'
+        for obj in ancestors
+        if obj["taxon_id"] in BUSCO_SETS
+    ]
+    lineages.extend(("bacteria_odb10", "archaea_odb10"))
     return lineages
 
 
-def fetch_file(url, filename):
+def fetch_file_curl(url, filename, protocol="ftp", retry_times=0):
     """fetch a remote file using curl."""
     filepath = Path(filename)
     if filepath.is_file():
         LOGGER.info("File exists, not overwriting")
         return
-    cmd = [
-        "curl",
-        "-L",
-        "-s",
-        "--connect-timeout",
-        "30",
-        "--retry",
-        "5",
-        "--retry-delay",
-        "5",
-        "-o",
-        "%s/%s" % (filepath.parent, filepath.name),
-        url,
-    ]
-    run(cmd)
+    if protocol:
+        url = url.replace("ftp:", f"{protocol}:").replace("ftps:", f"{protocol}:")
+    iteration = 0
+    complete = False
+    while iteration <= retry_times:
+        cmd = ["curl", "-L", "-s", "--connect-timeout", "30"]
+        if iteration:
+            LOGGER.info("Connection interrupted, resuming download")
+            cmd.extend(["-C", "-"])
+        cmd.extend(
+            [
+                "-o",
+                f"{filepath.parent}/{filepath.name}",
+                url,
+            ]
+        )
+        proc = run(cmd)
+        if proc.returncode == 0:
+            complete = True
+            break
+        iteration += 1
+    if not complete:
+        LOGGER.error(f"Unable to fetch {filename}")
+        sys.exit(1)
 
 
-def fetch_ncbi_datasets_assembly(url, filename, accession):
+def fetch_file_aria(url, filename, protocol="ftp", retry_times=0, binary="aria2c"):
+    """fetch a remote file using aria2."""
+    filepath = Path(filename)
+    if filepath.is_file():
+        LOGGER.info("File exists, not overwriting")
+        return
+    if protocol:
+        url = url.replace("ftp:", f"{protocol}:").replace("ftps:", f"{protocol}:")
+    iteration = 0
+    complete = False
+    while iteration <= retry_times:
+        cmd = [binary, "-q", "-d", filepath.parent]
+        if iteration:
+            LOGGER.info("Connection interrupted, resuming download")
+            cmd.append("-c")
+        cmd.extend(["-o", filepath.name, url])
+        proc = run(cmd)
+        if proc.returncode == 0:
+            complete = True
+            break
+        iteration += 1
+    if not complete:
+        LOGGER.error(f"Unable to fetch {filename}")
+        sys.exit(1)
+
+
+def fetch_ncbi_datasets_assembly(filename, accession, retry_times):
     """Fetch a remote assembly using NCBI datasets."""
     filepath = Path(filename)
     if filepath.is_file():
         LOGGER.info("File exists, not overwriting")
         return
-    dataset_zip = "%s/%s_dataset.zip" % (filepath.parents[0], accession)
+    dataset_zip = f"{filepath.parents[0]}/{accession}_dataset.zip"
     download_dehydrated = [
         "datasets",
         "download",
@@ -175,19 +215,32 @@ def fetch_ncbi_datasets_assembly(url, filename, accession):
         "--filename",
         dataset_zip,
     ]
-    run(download_dehydrated)
+    proc = run(download_dehydrated)
+    if proc.returncode == 0:
+        LOGGER.error(f"Unable to fetch {accession} with NCBI datasets")
+        sys.exit(1)
     with ZipFile(dataset_zip, "r") as zh:
         zh.extractall(filepath.parents[0])
     rehydrate = ["datasets", "rehydrate", "--directory", filepath.parents[0]]
-    run(rehydrate)
-    assembly_dir = "%s/ncbi_dataset/data/%s/" % (filepath.parents[0], accession)
-    LOGGER.info("Writing compressed file to %s" % filename)
-    with open(glob("%s/*_genomic.fna" % assembly_dir)[0], "rb") as ifh:
+    iteration = 0
+    complete = False
+    while iteration <= retry_times:
+        proc = run(rehydrate)
+        if proc.returncode == 0:
+            complete = True
+            break
+        iteration += 1
+    if not complete:
+        LOGGER.error(f"Unable to rehydrate {accession} with NCBI datasets")
+        sys.exit(1)
+    assembly_dir = f"{filepath.parents[0]}/ncbi_dataset/data/{accession}/"
+    LOGGER.info(f"Writing compressed file to {filename}")
+    with open(glob(f"{assembly_dir}/*_genomic.fna")[0], "rb") as ifh:
         with gzip.open(filename, "wb") as ofh:
             ofh.writelines(ifh)
     os.remove(dataset_zip)
-    os.remove("%s/README.md" % filepath.parents[0])
-    rmtree("%s/ncbi_dataset" % filepath.parents[0])
+    os.remove(f"{filepath.parents[0]}/README.md")
+    rmtree(f"{filepath.parents[0]}/ncbi_dataset")
 
 
 def fetch_assembly_url(accession, api_key=None):
@@ -198,11 +251,12 @@ def fetch_assembly_url(accession, api_key=None):
     if api_key and api_key is not None:
         eutils_env["NCBI_API_KEY"] = api_key
     esearch = Popen(
-        "esearch -db assembly -query %s" % accession,
+        f"esearch -db assembly -query {accession}",
         stdout=PIPE,
         shell=True,
         env=eutils_env,
     )
+
     esummary = Popen(
         "esummary", stdin=esearch.stdout, stdout=PIPE, shell=True, env=eutils_env
     )
@@ -214,28 +268,42 @@ def fetch_assembly_url(accession, api_key=None):
     )
     xtract_stdout = xtract.communicate()[0].decode("utf-8").strip()
     for url in xtract_stdout.split("\n"):
-        basenames = re.findall(GCA_NAME, url)
-        if basenames:
+        if basenames := re.findall(GCA_NAME, url):
             for basename in basenames:
                 if accession in basename:
-                    return "%s/%s_genomic.fna.gz" % (url, basename)
+                    return f"{url}/{basename}_genomic.fna.gz"
     return None
 
 
-def fetch_assembly_fasta(url, filename, accession):
+def fetch_assembly_fasta(
+    url, filename, accession, protocol, retry_times, download_client
+):
     """Save assembly fasta file to local disk."""
-    if which("datasets") is not None:
-        LOGGER.info("Fetching assembly FASTA to %s using NCBI datasets" % filename)
-        fetch_ncbi_datasets_assembly(url, filename, accession)
+    if download_client == "datasets" and which(download_client) is not None:
+        LOGGER.info(f"Fetching assembly FASTA to {filename} using NCBI datasets")
+        fetch_ncbi_datasets_assembly(filename, accession, retry_times)
+
+    elif download_client.startswith("aria") and which(download_client) is not None:
+        LOGGER.info(f"Fetching assembly FASTA to {filename} using {download_client}")
+        fetch_file_aria(url, filename, protocol, retry_times, download_client)
     else:
-        LOGGER.info("Fetching assembly FASTA to %s using curl" % filename)
-        fetch_file(url, filename)
+        LOGGER.info(f"Fetching assembly FASTA to {filename} using curl")
+        fetch_file_curl(url, filename, protocol, retry_times)
+
+
+def write_cat_file(data, filename):
+    result = [key for key, obj in data.items()]
+    arr = [result]
+    for idx, value in enumerate(data[result[0]]["list"]):
+        row = [value]
+        row.extend(data[key]["list"][idx] for key in result[1:])
+        arr.append(row)
+    tofile.write_file(filename, arr)
+    return result
 
 
 def parse_assembly_report(filename, cat_filename, syn_filename):
     """Parse synonyms and assembly level into tsv files."""
-    synonyms = []
-    categories = []
     cats = {
         "identifier": {"index": 4, "list": []},
         "assembly_role": {"index": 1, "list": []},
@@ -257,34 +325,16 @@ def parse_assembly_report(filename, cat_filename, syn_filename):
                 for obj in group.values():
                     value = row[obj["index"]]
                     obj["list"].append(value)
-    header = []
-    for key, obj in cats.items():
-        if len(set(obj["list"])) > 1:
-            header.append(key)
-    categories.append(header)
-    for idx, value in enumerate(cats[header[0]]["list"]):
-        row = [value]
-        for key in header[1:]:
-            row.append(cats[key]["list"][idx])
-        categories.append(row)
-    tofile.write_file(cat_filename, categories)
-    header = []
-    for key, obj in names.items():
-        if len(set(obj["list"])) > 1:
-            header.append(key)
-    synonyms.append(header)
-    for idx, value in enumerate(names[header[0]]["list"]):
-        row = [value]
-        for key in header[1:]:
-            row.append(names[key]["list"][idx])
-        synonyms.append(row)
-    tofile.write_file(syn_filename, synonyms)
+    write_cat_file(cats, cat_filename)
+    write_cat_file(names, syn_filename)
 
 
-def fetch_assembly_report(url, filename, cat_filename, syn_filename):
+def fetch_assembly_report(
+    url, filename, cat_filename, syn_filename, protocol, retry_times
+):
     """Save assembly report file to local disk."""
-    LOGGER.info("Fetching assembly report to %s" % filename)
-    fetch_file(url, filename)
+    LOGGER.info(f"Fetching assembly report to {filename}")
+    fetch_file_curl(url, filename, protocol, retry_times)
     parse_assembly_report(filename, cat_filename, syn_filename)
 
 
@@ -292,9 +342,7 @@ def fetch_assembly_meta_xml(accession):
     """
     Fetch assembly metadata xml from ENA.
     """
-    url = "%s/xml/%s" % (ENA_API, accession)
-    xml = tofetch.fetch_url(url)
-    return xml
+    return tofetch.fetch_url(f"{ENA_API}/xml/{accession}")
 
 
 def deep_find_text(data, tags):
@@ -306,7 +354,7 @@ def deep_find_text(data, tags):
     for tag in tags:
         try:
             data = data.find(tag)
-        except:
+        except Exception:
             return ""
     return data.text
 
@@ -362,7 +410,7 @@ def parse_assembly_meta(accession):
     wgs_prefix = deep_find_text(asm, ("WGS_SET", "PREFIX"))
     wgs_version = deep_find_text(asm, ("WGS_SET", "VERSION"))
     if wgs_prefix and wgs_version:
-        meta["assembly"]["prefix"] = "%s%s" % (wgs_prefix, wgs_version.zfill(2))
+        meta["assembly"]["prefix"] = f"{wgs_prefix}{wgs_version.zfill(2)}"
     elif " " not in meta["assembly"]["alias"]:
         meta["assembly"]["prefix"] = meta["assembly"]["alias"].replace(".", "_")
     else:
@@ -382,28 +430,28 @@ def fetch_busco_lineages(busco_sets, buscodir):
         return
     lineages_to_fetch = []
     for lineage in busco_sets:
-        busco_lineage = "%s/lineages/%s" % (buscodir, lineage)
+        busco_lineage = f"{buscodir}/lineages/{lineage}"
         if not os.path.isdir(busco_lineage):
             lineages_to_fetch.append(lineage)
     if not lineages_to_fetch:
         return
     lineage_urls = {}
     LOGGER.info("Fetching BUSCO lineage directory listing")
-    listing = tofetch.fetch_url("%s/" % BUSCO_URL)
+    listing = tofetch.fetch_url(f"{BUSCO_URL}/")
     for entry in listing.split("\n"):
         parts = re.split(r"[\"\s]+", entry)
         if len(parts) == 8:
             busco_set = re.sub(r"\..+$", "", parts[2])
-            lineage_urls.update({busco_set: "%s/%s" % (BUSCO_URL, parts[2])})
+            lineage_urls[busco_set] = f"{BUSCO_URL}/{parts[2]}"
     for lineage in lineages_to_fetch:
-        LOGGER.info("Fetching BUSCO lineage %s" % lineage)
+        LOGGER.info(f"Fetching BUSCO lineage {lineage}")
         tofetch.fetch_tar(lineage_urls[lineage], buscodir)
 
 
 def fetch_goat_data(taxon_id):
     """Fetch taxon metadata from GoaT."""
     LOGGER.info("Fetching taxon metadata")
-    url = "%s/record?recordId=taxon_id-%s&result=taxon" % (GOAT_API, taxon_id)
+    url = f"{GOAT_API}/record?recordId=taxon_id-{taxon_id}&result=taxon"
     result = tofetch.fetch_url(url)
     if result is None:
         LOGGER.error("Unable to fetch taxon metadata for '%s' from GoaT", taxon_id)
@@ -415,10 +463,8 @@ def fetch_goat_data(taxon_id):
 def fetch_read_info(accession, per_platform):
     """Fetch read info for an accession."""
     portal = "https://www.ebi.ac.uk/ena/portal/api"
-    url = (
-        "%s/filereport?accession=%s&result=read_run&fields=run_accession,fastq_bytes,base_count,library_strategy,library_selection,library_layout,instrument_platform,experiment_title,fastq_ftp"
-        % (portal, accession)
-    )
+    url = f"{portal}/filereport?accession={accession}&result=read_run&fields=run_accession,fastq_bytes,base_count,library_strategy,library_selection,library_layout,instrument_platform,experiment_title,fastq_ftp"
+
     data = tofetch.fetch_url(url)
     if data is None:
         return
@@ -432,14 +478,16 @@ def fetch_read_info(accession, per_platform):
         fields = line.split("\t")
         values = {}
         platform = "OTHER"
-        for i in range(0, len(header)):
+        for i in range(len(header)):
             value = fields[i]
             if header[i] == "instrument_platform" and platform != "ILLUMINA_XTEN":
                 platform = fields[i]
-            if header[i] == "experiment_title":
-                if value == "HiSeq X Ten paired end sequencing":
-                    platform = "ILLUMINA_XTEN"
-            values.update({header[i]: value})
+            if (
+                header[i] == "experiment_title"
+                and value == "HiSeq X Ten paired end sequencing"
+            ):
+                platform = "ILLUMINA_XTEN"
+            values[header[i]] = value
         if "base_count" in values:
             values["base_count"] = int(values["base_count"])
         else:
@@ -469,38 +517,31 @@ def assembly_reads(accession, read_runs, platforms):
     for acc in accession:
         fetch_read_info(acc, per_platform)
     for key in platforms.split(","):
-        arr = per_platform[key]
         runs = []
-        if arr:
-            for entry in arr:
-                runs.append(entry)
+        if arr := per_platform[key]:
+            runs.extend(iter(arr))
         runs = sorted(runs, key=itemgetter("base_count"), reverse=True)[
             : read_runs - len(sra)
         ]
         sra += runs
         if len(sra) >= read_runs:
             break
-    if sra:
-        return sra
-    return None
+    return sra or None
 
 
 def base_count(x):
     """Return number of bases or zero."""
-    if isinstance(x["base_count"], list):
-        return int(x["base_count"][0] or 0)
-    else:
-        return 0
+    return int(x["base_count"][0] or 0) if isinstance(x["base_count"], list) else 0
 
 
-def fetch_read_files(meta):
+def fetch_read_files(meta, protocol, retry_times):
     """Fetch sra reads."""
     files = meta["file"].split(";")
     for index, url in enumerate(meta["fastq_ftp"].split(";")):
-        url = "ftp://%s" % url
+        url = f"ftp://{url}"
         read_file = files[index]
         LOGGER.info("Fetching read file %s", read_file)
-        fetch_file(url, read_file)
+        fetch_file_curl(url, read_file, protocol, retry_times)
 
 
 def add_taxon_to_meta(meta, taxon_meta):
@@ -524,7 +565,7 @@ def add_taxon_to_meta(meta, taxon_meta):
 def add_reads_to_meta(meta, sra, readdir):
     """Add read accessions to metadata."""
     LOGGER.info("Adding read accessions to assembly metadata")
-    for index, library in enumerate(sra):
+    for library in sra:
         strategy = library["library_layout"].lower()
         fastq_ftp = library["fastq_ftp"].split(";")[-2:]
         info = {
@@ -532,10 +573,11 @@ def add_reads_to_meta(meta, sra, readdir):
             "platform": library["instrument_platform"],
             "base_count": library["base_count"],
             "file": ";".join(
-                [re.sub(r"^.+/", "%s/" % readdir, url) for url in fastq_ftp]
+                [re.sub(r"^.+/", f"{readdir}/", url) for url in fastq_ftp]
             ),
             "url": ";".join(fastq_ftp),
         }
+
         library["file"] = info["file"]
         meta["reads"][strategy].append(info)
 
@@ -544,69 +586,29 @@ def set_btk_version(meta):
     """Get curent version of hosted datasets from BTK API."""
     LOGGER.info("Checking current version on BTK public viewer")
     string = meta["assembly"]["prefix"]
-    btk = "https://blobtoolkit.genomehubs.org/api/v1/search/%s" % string
+    btk = f"https://blobtoolkit.genomehubs.org/api/v1/search/{string}"
     response = requests.get(btk)
     current = 0
     if response.ok:
         data = yaml.full_load(response.text)
         for asm in data:
-            if "version" in asm:
-                current = asm["version"]
-            else:
-                current = 1
+            current = asm["version"] if "version" in asm else 1
     meta["revision"] = current
     meta["version"] = current + 1
 
 
-# def create_outdir(span, version=1, _lineage="all"):
-#     """Create output directory."""
-#     span = tolkein.tobin.readable_bin(span)
-#     name = "%s/v%s/%s" % (OUTDIR, str(version), span)
-#     os.makedirs("%s/" % name, exist_ok=True)
-#     return name
-
-
-def main():
-    """Entry point."""
-    opts = docopt(__doc__)
-    accession = opts["<ACCESSION>"]
-    outdir = opts["--out"]
-    dbdir = opts["--db"]
-    buscodir = "%s/busco" % dbdir
-    uniprotdir = "%s/uniprot" % dbdir
-    ntdir = "%s/nt" % dbdir
-    taxdumpdir = "%s/taxdump" % dbdir
-    if opts["--db-suffix"]:
-        buscodir += "_%s" % opts["--db-suffix"]
-        ntdir += "_%s" % opts["--db-suffix"]
-        uniprotdir += "_%s" % opts["--db-suffix"]
-        taxdumpdir += "_%s" % opts["--db-suffix"]
-    if not outdir.endswith(accession):
-        outdir += "/%s" % accession
-    os.makedirs(outdir, exist_ok=True)
-    meta = parse_assembly_meta(accession)
-    assembly_url = fetch_assembly_url(accession, opts["--api-key"])
-    if assembly_url is None:
-        LOGGER.error("Unable to find assembly URL")
-        sys.exit(1)
-    assembly_file = "%s/assembly/%s.fasta.gz" % (outdir, accession)
-    meta["assembly"].update({"file": assembly_file, "url": assembly_url})
-    assembly_report = "%s/assembly/%s.report.txt" % (outdir, accession)
-    syn_filename = "%s/assembly/%s.synonyms.tsv" % (outdir, accession)
-    cat_filename = "%s/assembly/%s.categories.tsv" % (outdir, accession)
-    meta["fields"] = {
-        "synonyms": {"file": syn_filename, "prefix": "insdc"},
-        "categories": {"file": cat_filename},
-    }
+def process_reads(opts, outdir, protocol, retry_times, meta, sra):
+    if opts["--coverage"]:
+        meta["reads"].update({"coverage": {"max": int(opts["--coverage"])}})
+    readdir = f"{outdir}/reads"
+    add_reads_to_meta(meta, sra, readdir)
     if opts["--download"]:
-        os.makedirs(buscodir, exist_ok=True)
-        os.makedirs("%s/assembly" % outdir, exist_ok=True)
-        fetch_assembly_fasta(assembly_url, assembly_file, accession)
-        report_url = assembly_url.replace("_genomic.fna.gz", "_assembly_report.txt")
-        fetch_assembly_report(report_url, assembly_report, cat_filename, syn_filename)
-    taxon_meta = fetch_goat_data(meta["taxon"]["taxid"])
-    add_taxon_to_meta(meta, taxon_meta)
-    set_btk_version(meta)
+        os.makedirs(readdir, exist_ok=True)
+        for library in sra:
+            fetch_read_files(library, protocol, retry_times)
+
+
+def process_busco(opts, buscodir, meta, taxon_meta):
     busco_sets = find_busco_lineages(taxon_meta["lineage"])
     if busco_sets:
         meta["busco"].update(
@@ -622,26 +624,134 @@ def main():
         )
     if opts["--download"]:
         fetch_busco_lineages(busco_sets, buscodir)
+
+
+def download_assembly(
+    accession,
+    outdir,
+    protocol,
+    download_client,
+    retry_times,
+    buscodir,
+    assembly_url,
+    assembly_file,
+    assembly_report,
+    syn_filename,
+    cat_filename,
+):
+    os.makedirs(buscodir, exist_ok=True)
+    os.makedirs(f"{outdir}/assembly", exist_ok=True)
+    fetch_assembly_fasta(
+        assembly_url,
+        assembly_file,
+        accession,
+        protocol,
+        retry_times,
+        download_client,
+    )
+    report_url = assembly_url.replace("_genomic.fna.gz", "_assembly_report.txt")
+    fetch_assembly_report(
+        report_url,
+        assembly_report,
+        cat_filename,
+        syn_filename,
+        protocol,
+        retry_times,
+    )
+
+
+def set_defaults(opts):
+    accession = opts["<ACCESSION>"]
+    outdir = opts["--out"]
+    dbdir = opts["--db"]
+    protocol = opts["--protocol"]
+    download_client = opts["--download-client"]
+    retry_times = int(opts["--retry-times"])
+    buscodir = f"{dbdir}/busco"
+    uniprotdir = f"{dbdir}/uniprot"
+    ntdir = f"{dbdir}/nt"
+    taxdumpdir = f"{dbdir}/taxdump"
+    if opts["--db-suffix"]:
+        buscodir += f'_{opts["--db-suffix"]}'
+        ntdir += f'_{opts["--db-suffix"]}'
+        uniprotdir += f'_{opts["--db-suffix"]}'
+        taxdumpdir += f'_{opts["--db-suffix"]}'
+    if not outdir.endswith(accession):
+        outdir += f"/{accession}"
+    return (
+        accession,
+        outdir,
+        protocol,
+        download_client,
+        retry_times,
+        buscodir,
+        uniprotdir,
+        ntdir,
+        taxdumpdir,
+    )
+
+
+def main():
+    """Entry point."""
+    opts = docopt(__doc__)
+    (
+        accession,
+        outdir,
+        protocol,
+        download_client,
+        retry_times,
+        buscodir,
+        uniprotdir,
+        ntdir,
+        taxdumpdir,
+    ) = set_defaults(opts)
+    os.makedirs(outdir, exist_ok=True)
+    meta = parse_assembly_meta(accession)
+    assembly_url = fetch_assembly_url(accession, opts["--api-key"])
+    if assembly_url is None:
+        LOGGER.error("Unable to find assembly URL")
+        sys.exit(1)
+    assembly_file = f"{outdir}/assembly/{accession}.fasta.gz"
+    meta["assembly"].update({"file": assembly_file, "url": assembly_url})
+    assembly_report = f"{outdir}/assembly/{accession}.report.txt"
+    syn_filename = f"{outdir}/assembly/{accession}.synonyms.tsv"
+    cat_filename = f"{outdir}/assembly/{accession}.categories.tsv"
+    meta["fields"] = {
+        "synonyms": {"file": syn_filename, "prefix": "insdc"},
+        "categories": {"file": cat_filename},
+    }
+    if opts["--download"]:
+        download_assembly(
+            accession,
+            outdir,
+            protocol,
+            download_client,
+            retry_times,
+            buscodir,
+            assembly_url,
+            assembly_file,
+            assembly_report,
+            syn_filename,
+            cat_filename,
+        )
+    taxon_meta = fetch_goat_data(meta["taxon"]["taxid"])
+    add_taxon_to_meta(meta, taxon_meta)
+    set_btk_version(meta)
+    process_busco(opts, buscodir, meta, taxon_meta)
     read_accessions = []
     if meta["assembly"]["biosample"]:
         read_accessions = [meta["assembly"]["biosample"]]
     if opts["--reads"]:
         read_accessions += opts["--reads"]
-    sra = assembly_reads(read_accessions, int(opts["--read-runs"]), opts["--platforms"])
-    if sra:
-        if opts["--coverage"]:
-            meta["reads"].update({"coverage": {"max": int(opts["--coverage"])}})
-        readdir = "%s/reads" % outdir
-        add_reads_to_meta(meta, sra, readdir)
-        if opts["--download"]:
-            os.makedirs(readdir, exist_ok=True)
-            for library in sra:
-                fetch_read_files(library)
-    meta["similarity"]["blastn"].update({"path": ntdir})
-    meta["similarity"]["diamond_blastx"].update({"path": uniprotdir})
-    meta["similarity"]["diamond_blastp"].update({"path": uniprotdir})
+    if sra := assembly_reads(
+        read_accessions, int(opts["--read-runs"]), opts["--platforms"]
+    ):
+        process_reads(opts, outdir, protocol, retry_times, meta, sra)
+    meta["similarity"]["blastn"]["path"] = ntdir
+    meta["similarity"]["diamond_blastx"]["path"] = uniprotdir
+    meta["similarity"]["diamond_blastp"]["path"] = uniprotdir
     meta["settings"]["taxdump"] = taxdumpdir
-    tofile.write_file("%s/config.yaml" % outdir, meta)
+    tofile.write_file(f"{outdir}/config.yaml", meta)
 
 
 if __name__ == "__main__":
